
# q-expansion of newform 85.4.a.b, downloaded from the LMFDB on 26 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 85
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[3], [-5], [-5], [-22], [60], [-31], [17], [-61], [-78], [69], [-31], [56], [-6], [-538], [-465], [723], [-753], [35], [-322], [-99], [-1123], [488], [-852], [1215], [-601], [450], [1544], [276], [1739], [-657], [761], [-1074], [-1242], [-952], [-960], [2174], [866], [1712], [2202], [-414], [-4212], [1238], [-4236], [-274], [-156], [1775], [3158], [-3643], [-5703], [-3166], [5367], [-2394], [-142], [-5868], [4206], [7947], [-4737], [-5116], [-7462], [8619], [-343], [-3957], [-5884], [-8712], [-1966], [-6960], [1949], [-3643], [-4941], [-4912], [6690], [4530], [1406], [6518], [-8032], [-147], [5064], [9992], [-3384], [-10291], [8322], [5132], [15672], [6806], [13640], [17064], [2676], [9938], [-7302], [-8929], [2004], [-4041], [-10930], [-20235], [-1930], [9504], [-14148], [-4044], [1982], [16346], [-16309], [-22335], [16458], [-17877], [2438], [14612], [9798], [9834], [-9606], [12980], [-25270], [-7183], [-10377], [22538], [13436], [2610], [20756], [11061], [18858], [-6459], [-11332], [-6799], [-4350], [-19785], [-29668], [-13428], [11729], [31341], [10535], [-12070], [2537], [-15114], [-28285], [18269], [-7122], [-511], [8262], [30371], [4290], [-26166], [-6250], [-16713], [-12190], [30084], [-16516], [17055], [6662], [15], [-11077], [10488], [17372], [14154], [-34846], [42174], [31589], [-9144], [-45574], [31290], [-13420], [53361], [-6591], [20304], [-6280], [32943], [-35424], [40266], [-22327], [50762]]
